#ifndef CURL_FD_LEAK_TEST
#define CURL_FD_LEAK_TEST
#include <iostream>
#include <map>
#include <deque>
#include <vector>
#include <string>

#define VERBOSELOG

#ifdef VERBOSELOG
#define DLOG(x) x
#else
#define DLOG(x)
#endif


struct Doc {
	Doc() : url(), content() {}
	std::string url;
	std::vector<std::string>	headers;
	std::string content;
};



/**
 * @class Curl_handles
 * @brief Wrapper for curl_multi 
 */
class Curl_handles {
	friend std::ostream& operator<<(std::ostream& os, const Curl_handles&); 
	public:
		Curl_handles(int num_handles);
		~Curl_handles();
		/**
		 * Get curl messages with curl_multi_info_read and manage handler's work
		 */
		void process_handles();

		/**
		 * Set urls for the handlers to work
		 */
		void set_unused_to_work();

		void wait_events();
		void run();

		typedef enum handle_state_t {
			UNUSED,
			HEAD,
			BODY,
			FINISHED
		} handle_state_t;

		class Handle {
			friend std::ostream& operator<<(std::ostream& os, const Handle&); 
			public:
				Handle() : easy(0), doc(), state(UNUSED) { 
					DLOG(std::cout << "Handle(): &doc: " << (void*) &doc << std::endl;)
				}
				~Handle() {
					if( easy != 0 ) {
						DLOG(std::cout << "curl_easy_cleanup(" << easy << ")" << std::endl;)
						curl_easy_cleanup(easy);
					}	
				}
				CURL* easy;
				Doc doc;
				handle_state_t state;
		};


		std::deque<std::string> url_queue; // queue of urls to crawl

		/**
		 * Maps the easy handle to the Handle class that holds data
		 */
		typedef std::map<CURL*,boost::shared_ptr<Handle> > handles_t;
		handles_t handles;
		CURLM*	multi;
		int running;
	private:
		Curl_handles(const Curl_handles&);
		Curl_handles& operator=(const Curl_handles&);
};
#endif
